/*
 * Copyright 2010 Red Hat, Inc.
 * Red Hat licenses this file to you under the Apache License, version
 * 2.0 (the "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.hornetq.tests;

import java.util.ArrayList;

import org.hornetq.api.core.client.ClientConsumer;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.tests.util.ServiceTestBase;

public class BillsTest extends ServiceTestBase
{
   
   public void testLoop() throws Exception
   {
      for (int i = 0 ; i < 100; i++)
      {
         testSomething();
         tearDown();
         setUp();
      }
   }

   public void testSomething() throws Exception
   {
      HornetQServer server = createServer(true);
      try
      {
         server.start();

         ClientSessionFactory sf = createFactory(false);

         sf.setConsumerWindowSize(0);

         ClientSession session = sf.createSession(false, false, false);
         session.createQueue("testWindow", "testWindow", true);
         session.close();

         int numConsumers = 5;

         ArrayList<ClientConsumer> consumers = new ArrayList<ClientConsumer>();
         ArrayList<ClientSession> sessions = new ArrayList<ClientSession>();
         for (int i = 0; i < numConsumers; i++)
         {
            System.out.println("created: " + i);
            ClientSession session1 = sf.createSession();
            ClientConsumer consumer = session1.createConsumer("testWindow");
            consumers.add(consumer);
            session1.start();
            sessions.add(session1);
            // --- this is added later to cause error consumer.receiveImmediate();

         }
         
         ClientSession senderSession = sf.createSession(false, false);


         ClientProducer producer = senderSession.createProducer("testWindow");

         ClientMessage sent = senderSession.createMessage(true);
         sent.putStringProperty("hello", "world");

         producer.send(sent);
         
         senderSession.commit();
         
         senderSession.start();

         ClientConsumer consumer = consumers.get(2);
         ClientMessage received = consumer.receive(1000);
         assertNotNull(received);
         

         for (ClientSession tmpSess: sessions)
         {
            tmpSess.close();
         }
         
         senderSession.close();
         
      }
      finally
      {
         server.stop();
      }

   }

   // Constants -----------------------------------------------------

   // Attributes ----------------------------------------------------

   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------

   // Public --------------------------------------------------------

   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

}

